"""
Adapt from:
https://stackoverflow.com/a/14924210
https://stackoverflow.com/a/10415215

"""
import multiprocessing


def func_subprocess(func_, return_dict, dict_params):

    return_dict["return"] = func_(**dict_params)


def time_out(sec_time_out, func_, dict_params={}, restart=False):
    """Terminate the process if the runtime exceeds the defined time

    Args:
        sec_time_out (int or float): Maximum run time in seconds
        func_ (function): function to be performed
        dict_params (dict, optional): function parameters. Defaults to {}.
        restart (bool, optional): Restart the process if it is terminated by time_out. Defaults to False.

    Returns:
        type undefined: func_ return
    """

    # Starts shared variable
    manager = multiprocessing.Manager()
    return_dict = manager.dict()

    # Starts process
    p = multiprocessing.Process(
        target=func_subprocess, args=[func_, return_dict, dict_params]
    )
    p.start()

    # Wait for defined time  or until process finishes
    p.join(sec_time_out)

    # If thread is still active
    if p.is_alive():

        # Terminate - may not work if process is stuck for good
        # p.terminate()
        # OR Kill - will work for sure, no chance for process to finish nicely however
        p.kill()

        p.join()
        if restart:
            return_dict["return"] = time_out(
                sec_time_out, func_, dict_params, restart
            )
        else:
            pass

    return return_dict["return"]



from __future__ import annotations

import logging
import os
import sys
import time
from datetime import datetime
from pathlib import Path

from pyrogram import Client, types

from .. import utils
from ..mediainfo import ffprobe


def logging_config():
    log_file_name = "api_telegram"
    logfilename = "log-" + log_file_name + ".txt"
    logging.basicConfig(
        level=logging.warning,
        format=" %(asctime)s-%(levelname)s-%(message)s",
        handlers=[logging.FileHandler(logfilename, "w", "utf-8")],
    )
    # set up logging to console
    console = logging.StreamHandler()
    console.setLevel(logging.warning)
    # set a format which is simpler for console use
    formatter = logging.Formatter(" %(asctime)s-%(levelname)s-%(message)s")
    console.setFormatter(formatter)
    # add the handler to the root logger
    logging.getLogger("").addHandler(console)


async def get_telegram_app():

    app = Client("user")
    return await app.start()


async def ensure_connection():

    logging.warning("Ensuring connection...")
    if Path("user.session").exists():
        try:
            async with Client(
                "user", workdir=Path("user.session").absolute().parent
            ) as app:
                message = await app.send_message(
                    "me", text="telegram filesender-Connected..."
                )
                await message.delete()
            return
        except:
            print("Delete Session file and try again.")

    while True:
        try:
            api_id = int(input("Enter your api_id: "))
            api_hash = input("Enter your api_hash: ")

            async with Client(
                "user",
                api_id,
                api_hash,
                workdir=Path("user.session").absolute().parent,
            ) as app:
                message = await app.send_message(
                    "me", text="telegram filesender-Connected..."
                )
                await message.delete()
            return
        except:
            print("\nError. Try again.\n")
            pass


# Keep track of the progress while uploading
def progress(current, total):

    stringa = f"{current * 100 / total:.1f}%"
    sys.stdout.write(stringa)
    sys.stdout.flush()
    sys.stdout.write("\b" * len(stringa))
    sys.stdout.flush()


def get_video_metadata(file_path):

    try:
        metadata = ffprobe(file_path).get_output_as_dict()
        metadata_streams = metadata["streams"]
        video_metadata = metadata_streams[0]
    except:
        # case of error, show all file metadata
        print(file_path)
        print(ffprobe(file_path).get_output_as_dict())
        metadata = ffprobe(file_path).get_output_as_dict()
        metadata_streams = metadata["streams"]
        video_metadata = metadata_streams[0]
    try:
        width = video_metadata["width"]
    except:
        video_metadata = metadata_streams[1]
    try:
        width = video_metadata["width"]
        height = video_metadata["height"]
        duration = int(float(metadata["format"]["duration"]))
        return {"width": width, "height": height, "duration": duration}
    except Exception as e:
        logging.error(f"File Error: {file_path}.\napi_telegram.py line 63")
        raise ValueError(e)


async def send_video(app, chat_id, file_path, caption):

    logging.warning("Sending video...")

    video_metadata = get_video_metadata(file_path)
    thumb = utils.create_thumb(file_path)

    return_ = await app.send_video(
        chat_id,
        file_path,
        caption=caption,
        progress=progress,
        supports_streaming=True,
        width=video_metadata["width"],
        height=video_metadata["height"],
        duration=video_metadata["duration"],
        thumb=thumb,
    )
    os.remove(thumb)
    return return_


async def send_sticker(chat_id, sticker):

    logging.warning("Sending sticker...")
    async with Client(
        "user", workdir=Path("user.session").absolute().parent
    ) as app:
        return_ = await app.send_sticker(chat_id, sticker)
        return return_


async def send_audio(app, chat_id, file_path, caption):
    logging.warning("Sending audio...")
    return_ = await app.send_audio(
        chat_id, file_path, caption=caption, progress=progress
    )
    return return_


async def send_document(app, chat_id, file_path, caption):

    logging.warning("Sending document...")
    return_ = await app.send_document(
        chat_id, file_path, caption=caption, progress=progress
    )
    return return_


async def send_photo(chat_id, file_path, caption):

    logging.warning("Sending photo...")
    async with Client(
        "user", workdir=Path("user.session").absolute().parent
    ) as app:
        return_ = await app.send_photo(
            chat_id, file_path, caption=caption, progress=progress
        )
    return return_


async def send_message(chat_id, text):

    logging.warning("Sending message...")
    async with Client(
        "user", workdir=Path("user.session").absolute().parent
    ) as app:
        return_ = await app.send_message(
            chat_id, text=text, disable_web_page_preview=True
        )
    return return_


async def pin_chat_message(chat_id, message_id):

    logging.warning("Pinning message...")
    async with Client(
        "user", workdir=Path("user.session").absolute().parent
    ) as app:
        return_ = await app.pin_chat_message(
            chat_id, message_id=message_id, both_sides=True
        )
    return return_


async def get_messages(chat_id, message_ids):

    async with Client(
        "user", workdir=Path("user.session").absolute().parent
    ) as app:
        return_ = await app.get_messages(chat_id, message_ids)
    return return_


def get_history(chat_id):

    with Client("user", workdir=Path("user.session").absolute().parent) as app:
        return_ = app.get_history(chat_id)
    return return_


def get_list_media_doc(list_dict_sent_doc):
    """
    Generates a list of "inputmediadocument"
     from a list of 'return of sent files'.
    Used to forward files in album format with 'send_media_group' method

    Args:
        list_dict_sent_doc (list):
            List of returns from many files sent by the send_document method

    Returns:
        list: list of InputMediaDocument pyrogram type,
               necessary to send_media_group method
    """

    list_media_doc = []
    for dict_sent_doc in list_dict_sent_doc:
        file_id = dict_sent_doc["file_id"]
        caption = dict_sent_doc["caption"]
        media = types.InputMediaDocument(media=file_id, caption=caption)

        list_media_doc.append(media)
    return list_media_doc


async def send_media_group(chat_id, list_media):

    async with Client(
        "user", workdir=Path("user.session").absolute().parent
    ) as app:
        return_ = await app.send_media_group(chat_id, media=list_media)
    return return_


async def delete_messages(chat_id, list_message_id):

    async with Client(
        "user", workdir=Path("user.session").absolute().parent
    ) as app:
        return_ = await app.delete_messages(
            chat_id=chat_id, message_ids=list_message_id
        )
    return return_


async def send_file(app, dict_file_data, chat_id):

    file_path = dict_file_data["file_output"]
    description = dict_file_data["description"]
    file_extension = Path(file_path).suffix.lower()
    if file_extension == ".mp4":
        type_file = "video"
    elif file_extension == ".mp3":
        type_file = "audio"
    elif file_extension in [".png", ".jpg", ".jpeg", ".gif"]:
        type_file = "photo"
    else:
        type_file = "document"

    if type_file == "video":
        return_ = await send_video(
            app,
            chat_id=chat_id,
            file_path=file_path,
            caption=description,
        )
    elif type_file == "audio":
        return_ = await send_audio(
            app,
            chat_id=chat_id,
            file_path=file_path,
            caption=description,
        )
    elif type_file == "photo":
        return_ = await send_photo(
            app,
            chat_id=chat_id,
            file_path=file_path,
            caption=description,
        )
    elif type_file == "document":
        return_ = await send_document(
            app,
            chat_id=chat_id,
            file_path=file_path,
            caption=description,
        )

    return return_


async def send_files(list_dict, chat_id, time_limit=20):
    """Sends a series of files to the same chat_id

    Args:
        list_dict (list): list of dict. dict with keys:
            file_path=Absolute file_path
            description=file description
            file_output=file name for log
    """

    list_return = []
    len_list_dict = len(list_dict)

    app = await get_telegram_app()

    for index, d in enumerate(list_dict):
        order = index + 1
        file_path = d["file_output"]

        if not Path(file_path).exists():
            logging.error(f"file not exist. {file_path}")
            continue

        logging.warning(f"{order}/{len_list_dict} Uploading: {file_path}")

        file_extension = Path(file_path).suffix
        description = d["description"]

        if file_extension == ".mp4":
            type_file = "video"
        elif file_extension == ".mp3":
            type_file = "audio"
        elif file_extension in [".png", ".jpg", ".jpeg", ".gif"]:
            type_file = "photo"
        else:
            type_file = "document"

        while True:
            try:
                if type_file == "video":
                    return_ = await send_video(
                        app,
                        chat_id=chat_id,
                        file_path=file_path,
                        caption=description,
                    )
                elif type_file == "audio":
                    return_ = await send_audio(
                        app,
                        chat_id=chat_id,
                        file_path=file_path,
                        caption=description,
                    )
                elif type_file == "photo":
                    return_ = await send_photo(
                        app,
                        chat_id=chat_id,
                        file_path=file_path,
                        caption=description,
                    )
                elif type_file == "document":
                    return_ = await send_document(
                        app,
                        chat_id=chat_id,
                        file_path=file_path,
                        caption=description,
                    )
                break
            except Exception as e:
                print(e)
                print("\nError. Trying again...")
                time.sleep(30)
                continue
        list_return.append(return_)
    await app.stop()
    return list_return


async def create_channel(title, description):

    async with Client(
        "user", workdir=Path("user.session").absolute().parent
    ) as app:
        return_chat = await app.create_channel(
            title=title, description=description
        )
    chat_id = return_chat.id
    return chat_id


async def add_chat_members(chat_id, user_ids):

    async with Client(
        "user", workdir=Path("user.session").absolute().parent
    ) as app:
        return_chat = await app.add_chat_members(
            chat_id=chat_id, user_ids=user_ids
        )


async def promote_chat_members(chat_id, user_ids):

    async with Client(
        "user", workdir=Path("user.session").absolute().parent
    ) as app:

        privileges_config = types.ChatPrivileges(
            can_change_info=True,
            can_post_messages=True,
            can_edit_messages=True,
            can_delete_messages=True,
            can_promote_members=True,
        )

        for user_id in user_ids:
            await app.promote_chat_member(
                chat_id=chat_id, user_id=user_id, privileges=privileges_config
            )


async def set_chat_description(chat_id, description):

    async with Client(
        "user", workdir=Path("user.session").absolute().parent
    ) as app:
        await app.set_chat_description(
            chat_id=chat_id, description=description
        )


async def export_chat_invite_link(chat_id):

    async with Client(
        "user", workdir=Path("user.session").absolute().parent
    ) as app:
        return_ = await app.export_chat_invite_link(chat_id=chat_id)

    return return_


def get_template(folder_path_descriptions, file_name):

    template_path = Path(folder_path_descriptions) / file_name
    if not template_path.exists():
        template_path = (
            Path(__file__).absolute().parent.parent / "template" / file_name
        )
    if not template_path.exists():
        raise FileNotFoundError(f"{template_path=}")
    return template_path


def get_channel_title(folder_path_descriptions):

    header_project_name = "header_project.txt"
    header_project_path = get_template(
        folder_path_descriptions, header_project_name
    )

    channel_info_stringa = utils.get_txt_content(header_project_path)
    list_channel_info = channel_info_stringa.split("\n")

    title = list_channel_info[0]
    return title


def get_channel_description(chat_invite_link, folder_path_descriptions):

    header_template_name = "header_project.txt"
    header_template_path = get_template(
        folder_path_descriptions, header_template_name
    )

    channel_info_stringa = utils.get_txt_content(header_template_path)
    list_channel_info = channel_info_stringa.split("\n")

    description = "\n".join(list_channel_info[1:])
    description = description.replace("{chat_invite_link}", chat_invite_link)
    return description


def get_list_adms(channel_adms: str):

    if not channel_adms:
        return []
    channel_adms_stringa_list = channel_adms.split(",")
    list_adms = []
    for line in channel_adms_stringa_list:
        list_adms.append(line.strip())
    return list_adms



from __future__ import annotations

import asyncio
import logging
import os
import sys
import time
from configparser import ConfigParser
from pathlib import Path

import pandas as pd
import pyautogui as pag
import pyperclip

from . import api, api_async, utils


def get_config_data(path_file_config):
    """get default configuration data from file config.ini

    Returns:
        dict: config data
    """

    config_file = ConfigParser()
    config_file.read(path_file_config)
    default_config = dict(config_file["default"])
    return default_config


def gen_data_frame(list_file_path: list[Path]):

    list_dict = [
        {"description": file_path.name, "file_output": file_path}
        for file_path in list_file_path
    ]
    df = pd.DataFrame(list_dict)
    list_columns = ["file_output", "description"]
    df = df.reindex(list_columns, axis=1)
    return df


def create_report_descriptions(upload_plan_path):

    str_msg_paste_folder = "Paste the path folder with your files"
    str_msg_paste_here = "Paste here: "
    print(str_msg_paste_folder)
    while True:
        path_folder = input(str_msg_paste_here)
        if Path(path_folder).exists():
            all_file_path = utils.get_all_file_path(Path(path_folder))
            break
        else:
            logging.error("Folder not exist. Try again.")

    list_file_path = all_file_path["content"]
    df = gen_data_frame(list_file_path)
    df.to_csv(upload_plan_path, index=False)


def pag_hotkey(key_1, key_2):

    pag.keyDown(key_1)
    pag.keyDown(key_2)
    pag.keyUp(key_1)
    pag.keyUp(key_2)


def set_win_positions():
    """
    Keep the 'windows explorer' windows in front
    and telegram windows in back
    """

    pag_hotkey("alt", "tab")
    time.sleep(1)
    pag.keyDown("alt")
    pag.press("tab")
    time.sleep(0.5)
    pag.press("tab")
    pag.keyUp("alt")
    time.sleep(0.5)


def change_between_telegram_winexplorer():

    pag_hotkey("alt", "tab")
    time.sleep(2)


def get_next_video_to_send(path_file_description: Path):

    try:
        df = pd.read_csv(path_file_description)
    except Exception as e:
        print(f"Can't open file: {path_file_description}")
        print(e)

    mask_df_to_send = df["sent"].isin([0])
    df_to_send = df.loc[mask_df_to_send, :]
    count_files_to_send = df_to_send.shape[0]
    if count_files_to_send == 0:
        return 0, False
    else:
        file_index = df_to_send.index[0]
        df_to_send = df_to_send.reset_index(drop=True)
        dict_first_line = df_to_send.loc[0, :]
        return file_index, dict_first_line


def update_description_file_sent(path_file_description, dict_file_data):

    try:
        df = pd.read_csv(path_file_description)
    except Exception as e:
        print(f"Can't open file: {path_file_description}")
        print(e)

    file_path = dict_file_data["file_output"]
    mask_file_path = df["file_output"].isin([file_path])
    df_filter = df.loc[mask_file_path, :]
    len_df_filter = len(df_filter)
    if len_df_filter != 1:
        print(f"Find {len_df_filter} line for file: {file_path}")
        sys.exit()
    index_video = df_filter.index
    # update df
    df.loc[index_video, "sent"] = 1
    df.to_csv(path_file_description, index=False)


def ensure_existence_sent_column(df_list, file_path_descriptions):

    description_columns = df_list.columns
    if "sent" not in description_columns:
        df_list["sent"] = 0
        df_list.to_csv(file_path_descriptions, index=False)


def get_data_upload_plan(upload_plan_path_folder: Path) -> list[dict]:

    file_path_upload_plan = upload_plan_path_folder / "upload_plan.csv"
    df_upload_plan = pd.read_csv(file_path_upload_plan)
    data_upload_plan = df_upload_plan.to_dict("records")
    return data_upload_plan


def paste_on_telegram_app(desc):

    time.sleep(2)
    pag_hotkey("ctrl", "v")
    pyperclip.copy(desc)
    time.sleep(3)
    pag_hotkey("ctrl", "v")
    time.sleep(1)
    pag.press("enter")
    time.sleep(1)


def ask_send_app_or_api():

    str_msg_1 = "How do you intend to send the files?"
    str_msg_2 = "1-By telegram desktop app"
    str_msg_3 = "2-By telegram api (default)\n"
    str_msg_answer = "Type the number: "

    print("\n".join([str_msg_1, str_msg_2, str_msg_3]))

    answer = input(str_msg_answer)

    if answer == "":
        return 2

    return int(answer)


def send_via_telegram_app(data_upload_plan):

    set_win_positions()
    qt_files = len(data_upload_plan)
    print(f"Send {qt_files} files")
    time.sleep(0.5)

    for d in data_upload_plan:
        desc = d["description"]
        # copy file
        pag_hotkey("ctrl", "c")

        # change to telegram
        change_between_telegram_winexplorer()

        # paste on telegram app
        paste_on_telegram_app(desc)

        # change back to windows explorer and select next file
        change_between_telegram_winexplorer()
        pag.press("down")
        time.sleep(0.5)


async def process_create_channel(folder_path_descriptions):

    # TODO: The channel title must be the name of the original project folder
    title = api_async.get_channel_title(folder_path_descriptions)
    description = "channel description"

    chat_id = await api_async.create_channel(
        title=title, description=description
    )
    return chat_id


async def config_channel(
    chat_id, chat_invite_link, list_adms, folder_path_descriptions
):

    description = api_async.get_channel_description(
        chat_invite_link, folder_path_descriptions
    )
    await api_async.set_chat_description(
        chat_id=chat_id, description=description
    )

    if len(list_adms) != 0:
        await api_async.add_chat_members(chat_id=chat_id, user_ids=list_adms)
        await api_async.promote_chat_members(
            chat_id=chat_id, user_ids=list_adms
        )


def send_via_telegram_api(folder_path_upload_plan: Path, dict_config: dict):

    file_path_descriptions = folder_path_upload_plan / "upload_plan.csv"
    df_list = pd.read_csv(file_path_descriptions)

    ensure_existence_sent_column(df_list, file_path_descriptions)

    api.ensure_connection()

    chat_id = asyncio.run(
        process_to_send_telegram(folder_path_upload_plan, dict_config)
    )
    time_limit = int(dict_config["time_limit"])

    # Connection test to avoid infinite loop of asynchronous attempts
    # to send files without pause to fill connection pool

    files_count = df_list.shape[0]
    while True:
        index, dict_file_data = get_next_video_to_send(file_path_descriptions)
        # mark file as sent
        if dict_file_data is False:
            break
        else:
            file_path = dict_file_data["file_output"]
            logging.warning(f"{index+1}/{files_count} Uploading: {file_path}")

            try:
                api.send_file(dict_file_data, chat_id, time_limit)
            except Exception as e:
                logging.error("pyrogram error. %s", e)
                logging.warning("Trying again send file...")

            update_description_file_sent(
                file_path_descriptions, dict_file_data
            )


def test_chat_id(dict_config):

    if "chat_id" in dict_config.keys():
        chat_id = int(dict_config["chat_id"])
        is_int = isinstance(chat_id, int)
        if is_int:
            if chat_id < 0:
                return True
        print("config['chat_id'] must be negative integer")
        return False
    else:
        print("config['chat_id'] key 'chat_id' not found in config file")
        return False


def save_metadata_file(chat_id, chat_invite_link, file_path_metadata):

    dict_metadata = {"chat_id": chat_id, "chat_invite_link": chat_invite_link}
    utils.create_txt(file_path_metadata, str(dict_metadata))


async def process_to_send_telegram(folder_path_descriptions, dict_config):
    """
    Creates a new properly configured channel
    or use the existing chat_id in the configuration file

    Args:
        folder_path_descriptions (str):
            folder path where the upload_plan.csv file is located
        dict_config (dict):
            configuration data.
            template: {chat_id:negative int, create_new_channel:boolean}

    Returns:
        int: chat_id. negative integer
    """

    # check if there is necessary to continue upload
    file_path_metadata = os.path.realpath(
        os.path.join(folder_path_descriptions, "channel_metadata")
    )
    continue_upload = False
    if os.path.exists(file_path_metadata):
        dict_metadata = eval(utils.get_txt_content(file_path_metadata))
        continue_upload = True

    # get chat_id
    # Create new channel-Default True
    if dict_config["create_new_channel"] == "0" or continue_upload:
        channel_new = False
    else:
        channel_new = True

    if channel_new:
        # create new channel
        chat_id = await process_create_channel(folder_path_descriptions)
        time.sleep(5)
        chat_invite_link = await api_async.export_chat_invite_link(
            chat_id=chat_id
        )
        save_metadata_file(chat_id, chat_invite_link, file_path_metadata)

        # config new channel
        list_adms = api_async.get_list_adms(dict_config["channel_adms"])
        await config_channel(
            chat_id, chat_invite_link, list_adms, folder_path_descriptions
        )
    else:
        # use existent channel
        if continue_upload:
            dict_data = dict_metadata
        else:
            dict_data = dict_config

        chat_id_is_valid = test_chat_id(dict_data)
        if chat_id_is_valid:
            chat_id = int(dict_data["chat_id"])

    return chat_id


def ask_create_or_use(upload_plan_path):

    str_msg_create_or_use_1 = "About the upload_plan.csv report"
    str_msg_create_or_use_2 = "1-Use existing (default)"
    str_msg_create_or_use_3 = "2-Create a new one\n"
    str_msg_answer = "Type the number: "

    print(str_msg_create_or_use_1)
    print(str_msg_create_or_use_2)
    print(str_msg_create_or_use_3)

    create_or_use_answer = input(str_msg_answer)
    if create_or_use_answer == "2":
        create_report_descriptions(upload_plan_path)
        input("Report Created. Press a key to continue.")
    else:
        pass


def main():
    """
    An easy and automatic way to post a series of files to the telegram desktop
    app, along with personalized descriptions.

    How to use: https://github.com/apenasrr/Telegram_filesender/blob/master/README.md

    Args:
        upload_plan_path (str):
        dict_config (dict): {create_new_channel_flag: int, chat_id: int}
    """

    path_config_file = Path(__file__).absolute().parent / "config.ini"
    dict_config = get_config_data(path_config_file)

    upload_plan_path = Path("upload_plan.csv").absolute()
    ask_create_or_use(upload_plan_path)

    upload_plan_path_folder = upload_plan_path.parent
    data_upload_plan = get_data_upload_plan(upload_plan_path_folder)
    send_mode = ask_send_app_or_api()

    if send_mode == 1:
        send_via_telegram_app(data_upload_plan)
    elif send_mode == 2:
        send_via_telegram_api(upload_plan_path_folder, dict_config)


if __name__ == "__main__":

    main()



"""
source: https://stackoverflow.com/a/63215561/10681595
"""

import subprocess
import json
from pathlib import Path


class FFProbeResult:
    def __init__(
        self,
        return_code: int = None,
        output: str = "",
        error: str = "",
        format=None,
    ):
        self.return_code = return_code
        self.output = output
        self.error = error
        self.format = format
        self._output_as_dict = None

    def get_output_as_dict(self):
        if self._output_as_dict is None:
            if self.format == "json":
                self._output_as_dict = json.loads(self.output)
            elif self.format == "flat":
                output = [
                    e.split("=") for e in self.output.strip().split("\n")
                ]
                self._output_as_dict = {
                    key_val[0]: key_val[1] for key_val in output
                }
            else:
                raise ValueError(
                    "ffprobe format '%s' not supported to build dict"
                    % self.format
                )
        return self._output_as_dict

    def to_json_file(self, path: Path, mode="w", **kwargs):
        """
        :param mode: file open mode
        :param kwargs: kwargs for pathlib.Path().open()
        """
        path = path if isinstance(path, Path) else Path(path)
        with path.open(mode, **kwargs) as f:
            json.dump(self.get_output_as_dict(), f, indent=4)
            logger.debug("Dumped ffprobe output into %s", path)


def ffprobe(
    file_path, ffprobe_format="json", format_optn="", log_level="error"
) -> FFProbeResult:

    assert ffprobe_format in ["json", "flat"], (
        "format must be json or flat, not %s" % ffprobe_format
    )
    format_optn = "=" + format_optn if format_optn else format_optn
    command_array = [
        "ffprobe",
        "-v",
        log_level,
        "-print_format",
        ffprobe_format + format_optn,
        "-show_programs",
        "-show_format",
        "-show_streams",
        f"{file_path}",
    ]
    try:
        result = subprocess.run(
            command_array,
            stdout=subprocess.PIPE,
            stderr=subprocess.PIPE,
            universal_newlines=True,
            encoding="utf8",
        )
    except Exception as e:
        logger.critical(
            "ffprobe failed to run on %s, with the following error: '%s'\n"
            " check first that cmd is in your path",
            file_path,
            result.stderr,
            exc_info=True,
        )
        raise e

    return FFProbeResult(
        return_code=result.returncode,
        output=result.stdout,
        error=result.stderr,
        format=ffprobe_format,
    )
